/***************************************************************************
 * Copyright 2012 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.web.beans.view;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import kieker.webgui.common.exception.DataAccessException;
import kieker.webgui.domain.User;
import kieker.webgui.domain.User.Role;
import kieker.webgui.service.IUserService;
import kieker.webgui.web.beans.application.GlobalPropertiesBean;

/**
 * @author Nils Christian Ehmke
 */
@Component
@Scope("view")
public final class CurrentUserManagementBean {

	@Autowired
	private IUserService userService;

	private List<User> availableUses = new ArrayList<User>();

	/**
	 * Default constructor. <b>Do not call this constructor manually. It will only be accessed by Spring.</b>
	 */
	public CurrentUserManagementBean() {
		// No code necessary
	}

	/**
	 * This method initializes the bean. <b>Do not call this method manually. It will only be accessed by Spring.</b>
	 */
	@PostConstruct
	protected void initialialize() {
		this.updateList();
	}

	/**
	 * This method adds the given user to the database and informs about success via the growl component.
	 * 
	 * @param username
	 *            The name of the new user.
	 * @param password
	 *            The password of the new user.
	 * @param role
	 *            The role of the new user.
	 * @param isEnabled
	 *            Determines whether the user is enabled already or not.
	 */
	public void addUser(final String username, final String password, final Role role, final boolean isEnabled) {
		final User user = new User(username, password, role, isEnabled);

		try {
			this.userService.addUser(user);
			this.availableUses.add(user);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_INFO, "Added user to the database.");
		} catch (final DataAccessException ex) {
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, "Could not add the user to the database.");
		}
	}

	/**
	 * This method removed the given user from the database and informs about success via the growl component.
	 * 
	 * @param user
	 *            The user to be deleted.
	 */
	public void deleteUser(final User user) {
		try {
			this.userService.deleteUser(user);
			this.availableUses.remove(user);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_INFO, "Deleted user from the database.");
		} catch (final DataAccessException ex) {
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, "Could not deleted the user from the database.");
		}
	}

	/**
	 * This method edits the given user within the database and informs about success via the growl component.
	 * 
	 * @param user
	 *            The user to be modified.
	 */
	public void editUser(final User user) {
		try {
			this.userService.editUser(user);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_INFO, "Modified user within the database.");
		} catch (final DataAccessException ex) {
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, "Could not modified the user within the database.");
		}
	}

	/**
	 * Updates the list containing the available users.
	 */
	public void updateList() {
		try {
			this.availableUses = this.userService.getUsers();
		} catch (final DataAccessException ex) {
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, "An error occured while accessing the database.");
		}
	}

	/**
	 * This method delivers all available users.
	 * 
	 * @return All currently available users.
	 */
	public List<User> getUsers() {
		return this.availableUses;
	}
}
